-------------------------------------------------------------
-- 7-segment decoder
-------------------------------------------------------------
--
--  segment layout (alphabet appear segment's name.)
--
--        A
--      F   B
--        G
--      E   C
--        D
--
library ieee;
use ieee.std_logic_1164.all;

entity dec7Seg is
        port(   d    : in std_logic_vector(3 downto 0);
                segA : out std_logic;
                segB : out std_logic;
                segC : out std_logic;
                segD : out std_logic;
                segE : out std_logic;
                segF : out std_logic;
                segG : out std_logic);
end dec7Seg;

architecture rtl of dec7Seg is

        signal segBuf : std_logic_vector(6 downto 0);

begin
        with d select
             segBuf <=  "0111111" when "0000",  -- '0'
                        "0000110" when "0001",  -- '1'
                        "1011011" when "0010",  -- '2'
                        "1001111" when "0011",  -- '3'
                        "1100110" when "0100",  -- '4'
                        "1101101" when "0101",  -- '5'
                        "1111101" when "0110",  -- '6'
                        "0000111" when "0111",  -- '7'
                        "1111111" when "1000",  -- '8'
                        "1101111" when "1001",  -- '9'
                        "0000000" when others;

        segG <= segBuf(6);
        segF <= segBuf(5);
        segE <= segBuf(4);
        segD <= segBuf(3);
        segC <= segBuf(2);
        segB <= segBuf(1);
        segA <= segBuf(0);

end rtl;
