--
-- increment library
--
library ieee;
use ieee.std_logic_1164.all;
package libInc is
        function increment6(source : std_logic_vector(5 downto 0)) return std_logic_vector;
end libInc;

package body libInc is
        function increment6(source : std_logic_vector(5 downto 0)) return std_logic_vector is
                variable andAll : std_logic;
                variable temp   : std_logic_vector(source'high downto source'low);
        begin
                andAll := '1';
                for j in source'low to source'high loop
                        temp(j) := source(j) xor andAll;
                        andAll := andAll and source(j);
                end loop;
                return(temp);
        end increment6;
end libInc;

--
-- add library
--
library ieee;
use ieee.std_logic_1164.all;

package libAdd is
        function add4(aa,bb : std_logic_vector(3 downto 0)) return std_logic_vector;
end libAdd;

package body libAdd is
        function add4(aa,bb : std_logic_vector(3 downto 0)) return std_logic_vector is
                variable interCarry : std_logic;
                variable temp       : std_logic_vector(aa'high + 1 downto aa'low);
        begin
                interCarry := '0';
                for j in aa'low to aa'high loop
                        temp(j)    := aa(j) xor bb(j) xor interCarry;
                        interCarry := (aa(j) and bb(j)) or ( (aa(j) or bb(j)) and interCarry);
                end loop;
                temp(aa'high + 1) := interCarry;
                return(temp);
        end add4;
end libAdd;

--
-- +n counter with enable (use function)
--
-- data length : 10-bit  /  variable input : 4-bit
--
library ieee;
use ieee.std_logic_1164.all;
use work.libInc.all;
use work.libAdd.all;

entity count10_4var_func is
        port(   n  : in std_logic_vector(3 downto 0);
                e  : in std_logic;
                c  : in std_logic;
                nr : in std_logic;
                q  : out std_logic_vector(9 downto 0) );
end count10_4var_func;

architecture rtl of count10_4var_func is

        signal sum          : std_logic_vector(4 downto 0);
        signal interCarry   : std_logic;
        signal bufQIn       : std_logic_vector(9 downto 0);
        signal bufQ         : std_logic_vector(9 downto 0);
        signal bufQ9downto4 : std_logic_vector(5 downto 0);

begin
        process(nr,c)
        begin
                if (nr = '0') then
                        bufQ <= (others => '0');
                elsif (c'event and c = '1') then
                        if (e = '1') then
                                bufQ <= bufQIn;
                        end if;
                end if;
        end process;

        sum                <= add4(bufQ(3 downto 0),n);
        interCarry         <= sum(4);
        bufQ9downto4       <= bufQ(9 downto 4);
        bufQIn(9 downto 4) <= increment6(bufQ9downto4) when (interCarry = '1') else bufQ9downto4;
        bufQIn(3 downto 0) <= sum(3 downto 0);

        q <= bufQ;

end rtl;

