--
-- decrement library
--
library ieee;
use ieee.std_logic_1164.all;

package libDec is
        function decrement3(sorce : std_logic_vector(2 downto 0)) return std_logic_vector;
end libDec;

package body libDec is
        function decrement3(sorce : std_logic_vector(2 downto 0)) return std_logic_vector is
                variable orAll : std_logic;
                variable temp  : std_logic_vector(sorce'high downto sorce'low);
        begin
                orAll := '0';
                for j in sorce'low to sorce'high loop
                        temp(j) := sorce(j) xor (not orAll);
                        orAll := orAll or sorce(j);
                end loop;
                return(temp);
        end decrement3;
end libDec;

--
-- sequencer include loop control by loop counter
--
library ieee;
use ieee.std_logic_1164.all;
use work.libDec.all;

entity sqLoopB is
        port(   start : in std_logic;
                c     : in std_logic;
                nr    : in std_logic;
                st1   : out std_logic;
                st2   : out std_logic;
                st3   : out std_logic;
                st4   : out std_logic;
                st5   : out std_logic;
                st6   : out std_logic;
                st7   : out std_logic;
                monitLoopCount : out std_logic_vector(2 downto 0) );
end sqLoopB;

architecture rtl of sqLoopB is

        signal nonState : std_logic;
        signal st1In    : std_logic;
        signal st3In    : std_logic;
        signal st7In    : std_logic;
        signal stBuf    : std_logic_vector(7 downto 1);

        signal loopCount : std_logic_vector(2 downto 0);
        signal loopCountZero : std_logic;

begin

        process(nr,c)
        begin

                if (nr = '0') then
                        stBuf     <= (others => '0');
                        loopCount <= (others => '0');
                elsif (c'event and c = '1') then
                        stBuf(2 downto 1) <= stBuf(1) & st1In;
                        stBuf(6 downto 3) <= stBuf(5 downto 3) & st3In;
                        stBuf(7)          <= st7In;

                        if (stBuf(2) = '1') then
                                loopCount <= "100";
                        elsif (stBuf(4) = '1') then
                                loopCount <= decrement3(loopCount);
                        end if;
                end if;

        end process;

-- start
        nonState <= '1' when (stBuf = "0000000") else '0';
        st1In    <= nonState and start;
-- wait loop
        st3In <= stBuf(2) or (stBuf(6) and (not loopCountZero) );
-- loop out
        st7In <= stBuf(6) and loopCountZero;

        loopCountZero <= '1' when (loopCount = "000") else '0';

        st1 <= stBuf(1);
        st2 <= stBuf(2);
        st3 <= stBuf(3);
        st4 <= stBuf(4);
        st5 <= stBuf(5);
        st6 <= stBuf(6);
        st7 <= stBuf(7);

        monitLoopCount <= loopCount;

end rtl;
